@@ -1,6 +1,7 @@
 /*
  * Configuration file for CUPS on Windows.
  *
+ * Copyright © 2021-2023 by OpenPrinting
  * Copyright © 2007-2019 by Apple Inc.
  * Copyright © 1997-2007 by Easy Software Products.
  *
@@ -41,8 +42,10 @@
 #define open		_open
 #define read	        _read
 #define rmdir		_rmdir
+#define snprintf	_snprintf
 #define strdup		_strdup
 #define unlink		_unlink
+#define vsnprintf	_vsnprintf
 #define write		_write
 
 
@@ -97,8 +100,8 @@ typedef unsigned long useconds_t;
  * Version of software...
  */
 
-#define CUPS_SVERSION "CUPS v2.3.3"
-#define CUPS_MINIMAL "CUPS/2.3.3"
+#define CUPS_SVERSION "CUPS v2.4.4"
+#define CUPS_MINIMAL "CUPS/2.4.4"
 
 
 /*
@@ -126,6 +129,7 @@ typedef unsigned long useconds_t;
 
 #define CUPS_DEFAULT_LOG_LEVEL "warn"
 #define CUPS_DEFAULT_ACCESS_LOG_LEVEL "none"
+#define CUPS_DEFAULT_MAX_LOG_SIZE "1m"
 
 
 /*
@@ -159,11 +163,10 @@ typedef unsigned long useconds_t;
 
 
 /*
- * Default Samba and LPD config files...
+ * Default ErrorPolicy value...
  */
 
-#define CUPS_DEFAULT_SMB_CONFIG_FILE ""
-#define CUPS_DEFAULT_LPD_CONFIG_FILE ""
+#define CUPS_DEFAULT_ERROR_POLICY "stop-printer"
 
 
 /*
@@ -173,6 +176,13 @@ typedef unsigned long useconds_t;
 #define CUPS_DEFAULT_MAX_COPIES 9999
 
 
+/*
+ * Default SyncOnClose value...
+ */
+
+/* #undef CUPS_DEFAULT_SYNC_ON_CLOSE */
+
+
 /*
  * Do we have domain socket support, and if so what is the default one?
  */
@@ -198,7 +208,6 @@ typedef unsigned long useconds_t;
 #define CUPS_CACHEDIR "C:/CUPS/cache"
 #define CUPS_DATADIR "C:/CUPS/share"
 #define CUPS_DOCROOT "C:/CUPS/share/doc"
-#define CUPS_FONTPATH "C:/CUPS/share/fonts"
 #define CUPS_LOCALEDIR "C:/CUPS/locale"
 #define CUPS_LOGDIR "C:/CUPS/logs"
 #define CUPS_REQUESTS "C:/CUPS/spool"
@@ -330,9 +339,8 @@ typedef unsigned long useconds_t;
  * Do we have the (v)snprintf() functions?
  */
 
-/* Windows snprintf/vsnprintf are non-conforming */
-/* #undef HAVE_SNPRINTF */
-/* #undef HAVE_VSNPRINTF */
+#define HAVE_SNPRINTF 1
+#define HAVE_VSNPRINTF 1
 
 
 /*
@@ -377,10 +385,11 @@ typedef unsigned long useconds_t;
  * Which encryption libraries do we have?
  */
 
+#define HAVE_TLS 1
 /* #undef HAVE_CDSASSL */
+#define HAVE_OPENSSL 1
 /* #undef HAVE_GNUTLS */
-#define HAVE_SSPISSL 1
-#define HAVE_SSL 1
+/* #undef HAVE_SSPISSL */
 
 
 /*
@@ -422,24 +431,31 @@ typedef unsigned long useconds_t;
 
 
 /*
- * Do we have mDNSResponder for DNS Service Discovery (aka Bonjour)?
+ * Do we have DNS Service Discovery (aka Bonjour) support?
  */
 
 #define HAVE_DNSSD 1
 
 
 /*
- * Do we have Avahi for DNS Service Discovery (aka Bonjour)?
+ * Do we have mDNSResponder for DNS-SD?
  */
 
-#undef HAVE_AVAHI
+#define HAVE_MDNSRESPONDER 1
+
+
+/*
+ * Do we have Avahi for DNS-SD?
+ */
+
+/* #undef HAVE_AVAHI */
 
 
 /*
  * Do we have <sys/ioctl.h>?
  */
 
-#undef HAVE_SYS_IOCTL_H
+/* #undef HAVE_SYS_IOCTL_H */
 
 
 /*
